@@ -690,7 +690,7 @@ sub paste_clipboard {
 	my ($start, $end) = $self->get_selection_bounds;
 	if (defined $start) {
 		my $iter = $insert || $self->get_iter_at_mark( $self->get_insert );
-		if ($iter->in_range($start, $end)) {
+		if ($iter->in_range($start, $end) or $iter->equal($end)) {
 			$insert = $iter;
 			$replace = 1;
 		}
@@ -746,7 +746,7 @@ sub _clipboard_paste_data {
 	# check selection
 	$self->delete_selection(@{$paste_data}{'interactive','editable'})
 		if $paste_data->{replace};
-
+	
 	# check gtk_paste_point_override
 	my $iter;
 	my $mark = $self->get_mark('gtk_paste_point_override');
