@@ -2,9 +2,11 @@ package Zim;
 
 use strict;
 use vars qw/$AUTOLOAD %Config/;
+use File::BaseDir qw/xdg_config_home xdg_config_files xdg_cache_home/;
 use Gtk2;
 use Gtk2::Gdk::Keysyms;
 use Gtk2::SimpleList;
+use Zim::File;
 use Zim::History;
 use Zim::Components::TreeView;
 use Zim::Components::PathBar;
@@ -13,7 +15,7 @@ use Zim::Components::PageView;
 
 # TODO make components configable/autoloading like plugins
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 our $LONG_VERSION = << "EOT";
 zim $VERSION - A desktop wiki and outliner
 
@@ -266,7 +268,18 @@ Set the repository object.
 
 =cut
 
-sub set_repository { $_[0]->{repository} = $_[1] }
+sub set_repository {
+	my ($self, $repository) = @_;
+	
+	unless ($self->{name}) { # needed for trayicon tooltip
+		my @dirs = grep length($_),
+			File::Spec->splitdir( $repository->{dir} );
+			# FIXME what if there is no dir
+		$self->{name} = pop @dirs;
+	}
+
+	$_[0]->{repository} = $_[1]
+}
 
 =item C<gui_init()>
 
@@ -351,44 +364,13 @@ sub gui_init {
 	$vbox->pack_start($statusbar, 0,1,0);
 	$self->{statusbar} = $statusbar;
 	
-	## Build the content of the side pane
-#	my $l_top_hbox = Gtk2::HBox->new(0, 0);
-#	$l_vbox->pack_start($l_top_hbox, 0,1,0);
-	
-#	my $pane_combo = Gtk2::ComboBox->new_text;
-#	$pane_combo->signal_connect(changed =>
-#		sub { $self->show_in_pane($pane_combo->get_active_text()) } );
-#	# FIXME get_active_text is since 2.6
-#	$l_top_hbox->add($pane_combo);
-#	$self->{pane_combo} = $pane_combo;
-
-	my $close_button = Gtk2::Button->new;
-	$close_button->set_relief('none');
-	$close_button->add(Gtk2::Alignment->new(0.5, 0.5, 0, 0));
-	$close_button->child->add(
-		Gtk2::Image->new_from_stock('gtk-close', 'small-toolbar') );
-	$close_button->signal_connect(clicked =>
-		sub { $actions->get_action('TPane')->set_active(0) } );
-#	$l_top_hbox->pack_end($close_button, 0,1,0);
-	
-	my $notebook = Gtk2::Notebook->new;
-	$notebook->set_show_tabs(0);
-	$notebook->set_show_border(0);
-	$l_vbox->add($notebook);
-	$self->{notebook} = $notebook;
-	
+	## Build side pane
 	my $tree_view = Zim::Components::TreeView->new(app => $self);
 	$tree_view->signal_connect(row_activated =>
 		sub { $self->toggle_pane(0) if $self->{_pane_visible} == -1 } );
-	$self->add_to_pane($tree_view->widget, 'Index');
+	$l_vbox->add($tree_view->widget);
 	$self->{objects}{TreeView} = $tree_view;
 
-	#my $search_pane = Zim::Components::SearchPane->new(app => $self);
-	#$self->add_to_pane($search_pane->widget, 'Search');
-	#$self->{objects}{SearchPane} = $search_pane;
-	
-#	$pane_combo->set_active(0); # FIXME should be setting
-	
 	## Build content of the main area
 	my $path_bar = Zim::Components::PathBar->new(app => $self);
 	$r_vbox->pack_start($path_bar->widget, 0,1,5);
@@ -398,21 +380,26 @@ sub gui_init {
 	$r_vbox->add($page_view->widget);
 	$self->{objects}{PageView} = $page_view;
 
+	# FIXME shouldn't the repository manage this directory ?
 	if ($self->{settings}{use_xdg_cache}) {
-		my $root = File::Spec->rel2abs($self->{settings}{root_dir});
+		my $root = File::Spec->rel2abs($self->{repository}{dir});
 		my ($vol, $dirs) = File::Spec->splitpath($root, 'NO_FILE');
 		my @dirs = File::Spec->splitdir($dirs);
 		$self->{settings}{hist_file} ||= File::Spec->catfile( 
-			File::BaseDir->xdg_cache_home, 'zim', join ':', $vol, @dirs );
+			xdg_cache_home, 'zim', join ':', $vol, @dirs );
 	}
 	else {
-		my $root = File::Spec->rel2abs($self->{settings}{root_dir});
+		my $root = File::Spec->rel2abs($self->{repository}{dir});
 		$self->{settings}{hist_file} ||= File::Spec->catfile($root, '.zim.history'); 
 	}
-	my $history = Zim::History->new(
-		$self->{settings}{hist_file}, $page_view,
-		$self->{settings}{hist_max} );
-	$self->{objects}{History} = $history;
+	
+	eval {
+		my $history = Zim::History->new(
+			$self->{settings}{hist_file}, $page_view,
+			$self->{settings}{hist_max} );
+		$self->{objects}{History} = $history;
+	};
+	$self->exit_error($@) if $@;
 
 	## Some wiring
 	$ui->get_accel_group->connect( # Ctrl-Space
@@ -448,9 +435,12 @@ sub gui_init {
 			my $trayicon = Zim::Components::TrayIcon->new(app => $self);
 			$self->{objects}{TrayIcon} = $trayicon;
 		} ;
-		$self->error_dialog(
-			"Failed to load TrayIcon,\n".
-			"do you have Gtk2::TrayIcon installed ?", $@) if $@;
+		if ($@) {
+			$self->error_dialog(
+				"Failed to load TrayIcon,\n".
+				"do you have Gtk2::TrayIcon installed ?", $@);
+			$self->{settings}{use_tray_icon} = 0;
+		}
 	}
 	if ($self->{settings}{use_gtkspell}) {
 		eval {
@@ -458,9 +448,12 @@ sub gui_init {
 			my $spell = Zim::Components::Spell->new(app => $self);
 			$self->{objects}{Spell} = $spell;
 		} ;
-		$self->error_dialog(
-			"Failed to load spell checking,\n".
-			"do you have Gtk2::Spell installed ?", $@) if $@;
+		if ($@) {
+			$self->error_dialog(
+				"Failed to load spell checking,\n".
+				"do you have Gtk2::Spell installed ?", $@);
+			$self->{settings}{use_gtkspell} = 0;
+		}
 	}
 	if ($self->{settings}{use_calendar}) {
 		eval {
@@ -468,7 +461,10 @@ sub gui_init {
 			my $cal = Zim::Components::Calendar->new(app => $self);
 			$self->{objects}{Calendar} = $cal;
 		} ;
-		$self->error_dialog("Failed to load calendar", $@) if $@;
+		if ($@) {
+			$self->error_dialog("Failed to load calendar", $@);
+			$self->{settings}{use_calendar} = 0;
+		}
 	}
 
 	## Show all widgets .. well most of them
@@ -502,8 +498,8 @@ sub _gtk_action_set_sensitive { # **sigh**
 
 sub on_OpenRep {
 	my $self = pop;
-	my $rep = $self->prompt_repository_dialog;
-	$self->exec_new_window($rep) if defined $rep;
+	my ($name, $rep) = $self->prompt_repository_dialog;
+	$self->exec_new_window('--name', $name, $rep) if defined $rep;
 }
 
 sub on_Save { pop->save_page('FORCE') }
@@ -640,20 +636,12 @@ Read config file.
 sub load_config {
 	my $self = shift;
 	my $name = $self->{settings}{read_only} ? 'conf.ro' : 'conf.rw' ;
-	my ($file) =
-		grep { -f $_ && -r $_ }
-		map  { File::Spec->catdir($_, 'zim', $name) }
-		( File::BaseDir->xdg_config_home, File::BaseDir->xdg_config_dirs );
-
+	my ($file) = xdg_config_files('zim', $name);
 	return unless defined $file;
-	$self->{settings}{conf_file} = $file;
 	
-	open CONF, $file or $self->exit_error("Could not read\n$file");
-	while (<CONF>) {
-		/^(.+?)=(.*)$/ or next;
-		$self->{settings}{$1} = $2 if length $2;
-	}
-	close CONF;
+	my $conf = Zim::File->new($file)->read_config();
+	delete $$conf{$_} for grep {! length $$conf{$_}} keys %$conf;
+	%{$self->{settings}} = (%{$self->{settings}}, %$conf);
 }
 
 =item C<save_config>
@@ -666,18 +654,12 @@ sub save_config {
 	my $self = shift;
 	$self->{settings}{pane_pos} = $self->{hpaned}->get_position;
 	
-	my $dir  = File::Spec->catdir(
-		File::BaseDir->xdg_config_home, 'zim');
-	_mkdir($dir) unless -e $dir;
+	my $dir  = File::Spec->catdir(xdg_config_home, 'zim');
 	my $name = $self->{settings}{read_only} ? 'conf.ro' : 'conf.rw' ;
-	my $file = File::Spec->catfile($dir, $name);
-	
-	open CONF, ">$file" or $self->exit_error("Could not write config\n$file");
-	print CONF "# zim version:$Zim::VERSION\n";
-	print CONF "$_=$$self{settings}{$_}\n"
-		for sort keys %CONFIG; # we do not save all settings
-	close CONF;
-	#print "saved config to $file\n";
+
+	my %conf = %{$self->{settings}};
+	delete $conf{$_} for grep {! exists $CONFIG{$_}} keys %conf;
+	Zim::File->new($dir, $name)->write_config(\%conf);
 }
 
 sub _mkdir {
@@ -803,22 +785,7 @@ sub link_clicked {
 		unless length $link;
 
 	if ($link =~ /^(?:(\w+):\/\/|mailto:)/) { # link is an url
-		my $type = ($1 eq 'file') ? 'file_browser' : 'browser';
-		my $browser = $self->{settings}{$type};
-		$browser ||= $ENV{BROWSER} if $type eq 'browser';
-		$link =~ s#^file://## if $type eq 'file_browser';
-		unless ($browser) {
-			$browser = $self->prompt_browser_dialog($type);
-			return $self->error_dialog('You have no browser configured')
-				unless $browser;
-			$self->{settings}{$type} = $browser;
-		}
-		$browser =~ s/\%s/$link/ or $browser .= ' '.$link;
-		#warn "Executing: $browser\n";
-		unless (fork) { # child process
-			exec $browser;
-			exit 1; # just to be sure
-		}
+		$self->_ext_link_clicked($1, $link);
 	}
 	else {
 		$self->load_page(
@@ -826,6 +793,42 @@ sub link_clicked {
 	}
 }
 
+sub _ext_link_clicked {
+	my ($self, $proto, $link) = @_;
+	
+	my $type = ($proto eq 'file') ? 'file_browser' : 'browser';
+	my $browser = $self->{settings}{$type};
+	if ($type eq 'browser') { $browser ||= $ENV{BROWSER} }
+	else { $link =~ s#^file://## }
+	
+	unless ($browser) { # prompt for a browser
+		my ($title, $text, $value) = ($type eq 'file_browser')
+			? (
+				'Choose file browser',
+				'Please enter a file browser',
+				'rox \'%s\''
+			) : (
+				'Choose browser',
+				"Please enter a browser to open external links",
+				'firefox \'%s\''
+			) ;
+		my $val = $self->run_prompt(
+			$title, ['Command'], [$value], undef, undef, $text );
+		($browser) = @$val if $val;
+		$browser = undef unless $browser =~ /\S/;
+		return $self->error_dialog('You have no browser configured')
+			unless $browser;
+		$self->{settings}{$type} = $browser;
+	}
+	
+	$browser =~ s/\%s/$link/ or $browser .= ' '.$link;
+	#warn "Executing: $browser\n";
+	unless (fork) { # child process
+		exec $browser;
+		exit 1; # just to be sure
+	}
+}
+
 =item C<go_back(INT)>
 
 Go back one or more steps in the history stack.
@@ -892,10 +895,11 @@ sub load_page {
 		
 		eval { $page = $self->{repository}->get_page($name) };
 		return $self->error_dialog("Could not load page $name", $@) if $@;
-		return $self->error_dialog("Page does not exist:\n$page")
-			if ! defined $page
-			or $self->{settings}{read_only} and ! $page->exists ;
 	}
+	return $self->error_dialog("Page does not exist:\n$page")
+		if ! defined $page
+		or $self->{settings}{read_only} and ! $page->exists ;
+	
 	unless ($rec) { # get history record
 		$rec = $self->{objects}{History}->get_record($name);
 	}
@@ -960,9 +964,12 @@ sub save_page {
 		$self->signal_emit('page_saved', $page->name);
 	}
 	else { # FIXME this dialog could be a lot better
-		my $page = $self->prompt_page_dialog($page->name,
-			'Save As', 'Save Page As:', $@, 'gtk-dialog-error' );
-		return 0 unless defined $page;
+		my $val = $self->run_prompt(
+			'Save As', ['Save Page As'], [], 'gtk-save', undef,
+			'Could not save page, please select another name'  )
+			or return 0;
+		my ($page) = @$val;
+		return 0 unless $page =~ /\S/; 
 			# leave lock in position after returning
 		# FIXME check logic here !
 		$self->{page} = $self->{repository}->get_page($page);
@@ -1004,7 +1011,8 @@ sub rename_page {
 	#warn "Move '$from' to '$to'\n";
 	my $page;
 	eval { $page = $self->{repository}->move_page($from, $to) };
-	return $self->error_dialog("Could not move $from to $to\n\n$@") if $@;
+	return $self->error_dialog("Could not rename $from to $to\n\n$@") if $@;
+	# FIXME error reveal page object :(
 
 	# TODO trigger this with signals
 	# problem is that we might have moved a whole namespace
@@ -1136,11 +1144,101 @@ sub new_button {
 	$hbox->pack_start(
 		Gtk2::Label->new_with_mnemonic($text), 1,1,0);
 	my $button = Gtk2::Button->new();
-	$button->add(Gtk2::Alignment->new(0.5, 0.5, 0, 0));
+	$button->add(Gtk2::Alignment->new(0.5,0.5, 0,0));
 	$button->child->add($hbox);
 	return $button;
 }
 
+=item C<new_prompt(TITLE, FIELDS, DATA, BUTTON_STOCK, BUTTON_TEXT, TEXT)>
+
+Generates a dialog asking for one or more fields of input.
+Returns the dialog widget and a list with Gtk2::Entry objects.
+
+TITLE is the dialog title.
+
+FIELDS is an array ref containing the input fields.
+
+DATA is an array ref containing the text for the input fields.
+
+TEXT, BUTTON_STOCK and BUTTON_TEXT are optional.
+
+=cut
+
+sub new_prompt {
+	my ($self, $title, $fields, $data, $stock, $string, $text) = @_;
+	
+	## Setup dialog
+	my $dialog = Gtk2::Dialog->new(
+		$title, $self->{window},
+		[qw/modal destroy-with-parent no-separator/],
+		'gtk-cancel'  => 'cancel',
+	);
+	$dialog->set_resizable(0);
+	$dialog->set_border_width(5);
+	$dialog->set_icon($self->{window}->get_icon);
+	
+	$stock ||= 'gtk-ok';
+	my $button = $string
+		? $self->new_button($stock, $string)
+		: Gtk2::Button->new_from_stock($stock);
+	$dialog->add_action_widget($button, 'ok');
+	# $dialog->set_default_response('ok'); FIXME
+
+	if (defined $text) {
+		my $label = Gtk2::Label->new($text);
+		my $align = Gtk2::Alignment->new(0,0.5, 0,0);
+		$align->add($label);
+		$dialog->vbox->add($align);
+	}
+	
+	## Generate table with input fields
+	my $table = Gtk2::Table->new(scalar(@$fields), 2);
+	$table->set_border_width(5);
+	$table->set_row_spacings(5);
+	$table->set_col_spacings(12);
+	$dialog->vbox->add($table);
+	
+	my @entries;
+	for my $i (0 .. $#$fields) {
+		my $field = $$fields[$i];
+		
+		my $label = Gtk2::Label->new($field.':');
+		my $align = Gtk2::Alignment->new(0,0.5, 0,0);
+		$align->add($label);
+		
+		my $entry = Gtk2::Entry->new();
+		$entry->set_text($$data[$i]) if defined $$data[$i];
+		$entry->signal_connect(
+			activate => sub { $dialog->response('ok') } );
+		
+		$table->attach_defaults($align, 0,1, $i,$i+1);
+		$table->attach_defaults($entry, 1,2, $i,$i+1);
+		push @entries, $entry;
+	}
+	
+	$dialog->show_all;
+	return $dialog, \@entries;
+}
+
+=item C<run_prompt(..)>
+
+Wrapper around C<new_prompt()> that runs the dialog and
+returns a list with input values. Returns undef on 'cancel'.
+
+=cut
+
+sub run_prompt {
+	my $self = shift;
+	my ($dialog, $entries) = $self->new_prompt(@_);
+
+	my $values = ($dialog->run eq 'ok')
+		? [map $_->get_text, @$entries]
+		: undef ;
+	$dialog->destroy;
+
+	return $values;
+}
+
 =item toggle_pane(BOOLEAN)
 
 Show or hide the side pane with the index tree.
@@ -1156,7 +1254,7 @@ sub toggle_pane { # TODO does not update action state :(
 	if ($pane_visible) {
 		$hpaned->set_position($self->{settings}{pane_pos});
 		$widget->show_all;
-		$self->{notebook}->grab_focus;
+		$self->{objects}{TreeView}->grab_focus;
 	}
 	else {
 		$self->{settings}{pane_pos} = $hpaned->get_position();
@@ -1167,38 +1265,6 @@ sub toggle_pane { # TODO does not update action state :(
 		unless $pane_visible == -1;
 }
 
-=item C<add_to_pane(WIDGET, LABEL)>
-
-Add a widget to the side pane. The label is added to the menu at the top of the side pane.
-
-=cut
-
-sub add_to_pane {
-	my ($self, $widget, $label) = @_;
-	$self->{notebook}->append_page($widget, $label);
-#	$self->{pane_combo}->append_text($label);
-	# Cross you fingers, lets hope combobox and notebook count the same way
-	my $id = $self->{pane_items}{_id}++;
-	$self->{pane_items}{lc($label)} = $id;
-}
-
-=item C<show_in_pane(NAME)>
-
-Select which widget you want to see in the side pane.
-Opens the side pane if needed.
-NAME is the same string as the label used for C<add_to_pane()>.
-
-=cut
-
-sub show_in_pane {
-	my ($self, $label) = @_;
-	my $id = $self->{pane_items}{lc($label)};
-	die "No such widget: $label" unless defined $id;
-	$self->{notebook}->set_current_page($id);
-	$self->{pane_combo}->set_active($id);
-	#$self->{actions}->get_action('TPane')->set_active(1);
-}
-
 =item C<toggle_path_bar(TYPE)>
 
 Set or cycle the type of the pathbar.
@@ -1255,8 +1321,10 @@ This method is used to display errors.
 sub error_dialog {
 	my ($self, $text1, $text2) = @_;
 	$text2 ||= $@ || $text1;
+	my $window = $self->{window}->visible ? $self->{window} : undef;
+		# window might not yet be realized
 	my $dialog = Gtk2::MessageDialog->new(
-		$self->{window}, 'modal', 'error', 'ok', $text1 );
+		$window, 'modal', 'error', 'ok', $text1 );
 		# parent, flags, type, buttons, format, ...
 	print STDERR "zim: $text2\n";
 	$dialog->run;
@@ -1264,27 +1332,6 @@ sub error_dialog {
 	return undef;
 }
 
-=item C<repository_lock_dialog>
-
-=cut
-
-sub repository_lock_dialog {
-	my $self = shift;
-	my $dir = shift;
-	my $dialog = Gtk2::MessageDialog->new(
-		$self->{window}, 'modal', 'warning', 'yes-no', << "EOT");
-The directory $dir 
-seems to be already in use by
-an other instance of zim.
-
-Are you sure you want to continue ?
-EOT
-	$dialog->set_default_response('no');
-	my $ok = ($dialog->run eq 'yes') ? 1 : 0;
-	$dialog->destroy;
-	return $ok;
-}
-
 =item C<jump_page_dialog()>
 
 Prompts the user for a page name. This page is then opened.
@@ -1294,110 +1341,29 @@ Prompts the user for a page name. This page is then opened.
 sub jump_page_dialog {
 	my $self = shift;
 	my $page = $self->{page}->namespace;
-	my $button = Gtk2::Button->new_from_stock('gtk-jump-to');
-	$page = $self->prompt_page_dialog(
-		$page, 'Jump to', '_Jump to Page:', undef, undef, $button);
-	return unless defined $page;
-	$page = $self->{repository}->resolve_page($page);
-	$self->load_page($page);
-}
-
-=item C<prompt_page_dialog(PAGE, TITLE, PROMPT, TEXT, ICON)>
-
-Method used to prompt the user for a page name. Returns PAGE.
-
-The I<text> and I<icon> arguments are optional.
-
-=cut
-
-sub prompt_page_dialog {
-	my ($self, $page, $title, $prompt, $text, $icon, $button) = @_;
 
 	$page = '' if $page eq ':';
-	
-	my $dialog = Gtk2::Dialog->new(
-		$title, $self->{window},
-	       	[qw/modal destroy-with-parent no-separator/],
-		'gtk-cancel' => 'cancel',
-		( defined($button) ? () : ('gtk-open'   => 'ok') )
-	);
-	$dialog->set_resizable(0);
-	$dialog->set_border_width(5);
-	$dialog->set_icon($self->{window}->get_icon);
-	$dialog->add_action_widget($button, 'ok') if defined $button;
-	$dialog->set_default_response('ok') unless defined $button;
-		# the unless ... surpresses a warning ... how to put button as default ?
-
-	if (defined $text) {
-		my $text_hbox = Gtk2::HBox->new(0, 5);
-		$text_hbox->set_border_width(5);
-		$dialog->vbox->add($text_hbox);
+	my $values = $self->run_prompt(
+		'Jump to', ['Jump to Page'], [$page], 'gtk-jump-to')
+		or return;
+	($page) = @$values;
+	return unless $page =~ /\S/;
 		
-		if (defined $icon) {
-			my $image = Gtk2::Image->new_from_stock($icon, 'dialog');
-			$text_hbox->add($image);
-		}
-		
-		my $text_label = Gtk2::Label->new($text);
-		$text_hbox->add($text_label);
-	}
-	
-	my $hbox  = Gtk2::HBox->new(0, 5);
-	$hbox->set_border_width(5);
-	$dialog->vbox->add($hbox);
-	
-	my $label = Gtk2::Label->new_with_mnemonic($prompt);
-	$hbox->add($label);
-	
-	my $entry = Gtk2::Entry->new();
-	$entry->signal_connect(activate => sub { $dialog->response('ok') });
-	$entry->set_text($page);
-	$hbox->add($entry);
-	
-	#my $completion = Gtk2::EntryCompletion->new;
-	#$completion->set_model($self->TreeView->get_model());
-	#$completion->set_text_column(1);
-	#$entry->set_completion($completion);
-	
-	$dialog->show_all;
-	if ($dialog->run eq 'ok') { $page = $entry->get_text }
-	else                      { $page = undef            }
-	$dialog->destroy;
-
-	$page = undef unless $page =~ /\S/;
-	return $page;
+	$self->link_clicked($page);
 }
 
 =item C<prompt_rename_page_dialog>
 
 =cut
 
-sub prompt_rename_page_dialog { # TODO base this dialog on prompt_page_dialog()
+sub prompt_rename_page_dialog {
 	my ($self, $page) = @_;
 	
-	my $title = "Rename page";
-	my $dialog = Gtk2::Dialog->new(
-		$title, $self->{window},
-	       	[qw/modal destroy-with-parent no-separator/],
-		'gtk-cancel' => 'cancel',
-	);
-	$dialog->set_resizable(0);
-	$dialog->set_border_width(5);
-	$dialog->set_icon($self->{window}->get_icon);
-	$dialog->set_default_response('ok');
-
-	my $move_button = $self->new_button('gtk-save', '_Rename');
-	$dialog->add_action_widget($move_button, 'ok');
-	# $dialog->set_default_response('ok'); FIXME
-
-	my $entry = Gtk2::Entry->new();
-	$entry->signal_connect(activate => sub { $dialog->response('ok') });
-	$entry->set_text($page);
-	$dialog->vbox->add($entry);
-
-	$dialog->show_all;
+	my ($dialog, $entries) = $self->new_prompt(
+		'Rename page', ['Rename to'], [$page], 'gtk-save', '_Rename');
+	
+	my ($entry) = @$entries;
 	$entry->select_region(length($1), -1) if $page =~ /^(.+:)/;
-
 	if ($dialog->run eq 'ok') { $page = $entry->get_text }
 	else                      { $page = undef            }
 	$dialog->destroy;
@@ -1406,67 +1372,6 @@ sub prompt_rename_page_dialog { # TODO base this dialog on prompt_page_dialog()
 	return $page;
 }
 
-=item C<prompt_link_dialog(TEXT, LINK, TITLE)>
-
-Method used to prompt the user for a link. Returns TEXT and TITLE.
-
-=cut
-
-sub prompt_link_dialog {
-	my ($self, $text, $link, $title) = @_;
-	
-	my $dialog = Gtk2::Dialog->new(
-		$title, $self->{window},
-	       	[qw/modal destroy-with-parent no-separator/],
-		'gtk-cancel'  => 'cancel',
-	);
-	$dialog->set_resizable(0);
-	$dialog->set_border_width(5);
-	$dialog->set_icon($self->{window}->get_icon);
-	
-	my $link_button = $self->new_button($LINK_ICON, '_Link');
-	$dialog->add_action_widget($link_button, 'ok');
-	# $dialog->set_default_response('ok'); FIXME
-
-	my $table = Gtk2::Table->new(2, 2);
-	$table->set_border_width(5);
-	$table->set_row_spacings(5);
-	$table->set_col_spacings(5);
-	$dialog->vbox->add($table);
-	
-	my $text_label = Gtk2::Label->new('Text:');
-	my $link_label = Gtk2::Label->new('Links to:');
-	my $text_entry = Gtk2::Entry->new();
-	$text_entry->signal_connect(activate => sub { $dialog->response('ok') });
-	my $link_entry = Gtk2::Entry->new();
-	$link_entry->signal_connect(activate => sub { $dialog->response('ok') });
-	$table->attach_defaults($text_label, 0,1, 0,1);
-	$table->attach_defaults($text_entry, 1,2, 0,1);
-	$table->attach_defaults($link_label, 0,1, 1,2);
-	$table->attach_defaults($link_entry, 1,2, 1,2);
-	
-	$link_entry->set_text($link) if defined $link;
-	if (defined $text) {
-		$text_entry->set_text($text);
-		$link_entry->grab_focus;
-	}
-	else { $text_entry->grab_focus }
-	
-	$dialog->show_all;
-	if ($dialog->run eq 'ok') {
-		$text = $text_entry->get_text;
-		$link = $link_entry->get_text;
-		$dialog->destroy;
-		return undef
-			unless $text =~ /\S/ or $link =~ /\S/;
-		return $text, $link;
-	}
-	else {
-		$dialog->destroy;
-		return undef;
-	}
-}
-
 =item C<prompt_confirm_delete_page(PAGE)>
 
 Ask the user if they are sure about deleting a page.
@@ -1598,7 +1503,17 @@ sub filechooser_dialog {
 	#}
 	$dialog->set_icon($self->{window}->get_icon) if $self->{window};
 	$dialog->set_action('select-folder') if $dir;
-	$dialog->set_filename(File::Spec->rel2abs($file)) if defined $file;
+	if (defined $file) {
+		$file =~ s#^file://##;
+		$dialog->set_filename(File::Spec->rel2abs($file));
+	}
+	elsif (defined $self->{page}) {
+		my $dir = $self->{page}->properties->{base};
+		if (defined $dir) {
+			$dir =~ s#^file://##;
+			$dialog->set_current_folder(File::Spec->rel2abs($dir));
+		}
+	}
 	$dialog->signal_connect('response', sub {
 		$file = $_[1] eq 'ok' ? $dialog->get_filename : undef;
 		$dialog->destroy;
@@ -1626,6 +1541,7 @@ sub prompt_repository_dialog { # TODO improve visual layout
 	$dialog->set_icon(
 		Gtk2::Gdk::Pixbuf->new_from_file($self->{settings}{icon_file}) );
 	$dialog->set_default_response('ok');
+	local $self->{window} = $dialog; # parent might not yet be initialized
 	
 	$dialog->vbox->set_spacing(5);
 	$dialog->vbox->pack_start(
@@ -1637,20 +1553,12 @@ sub prompt_repository_dialog { # TODO improve visual layout
 	$list->signal_connect(row_activated => sub { $dialog->response('ok') });
 	$dialog->vbox->pack_start($list, 1,1,0);
 	
-	# Read list - FIXME also look in other xdg_config dirs
-	my %source;
-	my $file = File::Spec->catfile(
-		File::BaseDir->xdg_config_home, 'zim', 'repositories.list' );
-	if (-f $file) {
-		open IN, $file or die "Could not read file: $file\n";
-		while (<IN>) {
-			chomp;
-			my ($name, $source) = split '=', $_, 2;
-			next unless $name =~ /\S/;
-			$source{$name} = $source;
-			push @{$list->{data}}, $name;
-		}
-		close IN;
+	my ($source, $order) = ({}, []);
+	my ($file) = xdg_config_files('zim', 'repositories.list');
+	if (defined $file) {
+		$file = Zim::File->new($file);
+		($source, $order) = $file->read_config;
+		@{$list->{data}} = @$order;
 	}
 	
 	my $hbox = Gtk2::HBox->new(1,10);
@@ -1662,9 +1570,9 @@ sub prompt_repository_dialog { # TODO improve visual layout
 	
 	my $modified = 0;
 	$add_button->signal_connect(clicked => sub {
-			my ($name, $source) = $self->prompt_new_repository_dialog($dialog);
+			my ($name, $dir) = $self->prompt_new_repository_dialog();
 			return unless defined $name;
-			$source{$name} = $source;
+			$$source{$name} = $dir;
 			push @{$list->{data}}, $name;
 			$modified = 1;
 		} );
@@ -1675,33 +1583,33 @@ sub prompt_repository_dialog { # TODO improve visual layout
 		} );
 	
 	$dialog->show_all;
-	my $repository;
+	my ($name, $repository);
 	RUN_REP_DIALOG:
 	if ($dialog->run eq 'ok') {
 		my ($i) = $list->get_selected_indices;
 		unless(defined $i) {
-			$self->error_dialog("Please select a repository first.\n");
-			goto RUN_REP_DIALOG;
+			if (@{$list->{data}} == 1) { $i = 0 }
+			else {
+				$self->error_dialog("Please select a repository first.\n");
+				goto RUN_REP_DIALOG;
+			}
 		}
 		
-		my ($name) = @{$list->{data}[$i]};
-		$repository = $source{$name};
+		($name) = @{$list->{data}[$i]};
+		$repository = $$source{$name};
 		#warn "You selected \"$name\" ($i) => $repository\n";
 	}
-	else { $modified = 0 } # Don't save at cancel
 	$dialog->destroy;
 
 	# Save list
 	if ($modified) {
-		open OUT, ">$file" or die "Could not write file: $file\n";
-		for (@{$list->{data}}) {
-			my ($name) = @$_;
-			print OUT "$name=$source{$name}\n";
-		}
-		close OUT or die "Could not write file: $file\n";
+		@$order = map $$_[0], @{$list->{data}};
+		$file = Zim::File->new(
+			xdg_config_home, 'zim', 'repositories.list' );
+		$file->write_config($source, $order);
 	}
 	
-	return $repository;
+	return ($name, $repository);
 }
 
 =item C<prompt_new_repository_dialog(WINDOW)>
@@ -1710,94 +1618,18 @@ sub prompt_repository_dialog { # TODO improve visual layout
 
 sub prompt_new_repository_dialog { # TODO add "browse" button for dir
 	my $self = shift;
-	my $window = shift || $self->{window};
+	my $val = $self->run_prompt(
+		'New repository - Zim', ['Directory', 'Name'], [], undef, undef,
+		'Please give a directory to store your pages' )
+		or return undef;
 	
-	my $dialog = Gtk2::Dialog->new(
-		'New repository - Zim', $window,
-	       	[qw/modal destroy-with-parent no-separator/],
-		'gtk-cancel' => 'cancel',
-		'gtk-ok'     => 'ok',
-	);
-	$dialog->set_resizable(0);
-	$dialog->set_border_width(5);
-	$dialog->set_icon(
-		Gtk2::Gdk::Pixbuf->new_from_file($self->{settings}{icon_file}) );
-	$dialog->set_default_response('ok');
-
-	$dialog->vbox->set_spacing(5);
-	$dialog->vbox->add( Gtk2::Label->new(
-		"Please give a new directory\nto store your pages" ) );
-	
-	my $table = Gtk2::Table->new(2, 2);
-	$table->set_border_width(5);
-	$table->set_row_spacings(5);
-	$table->set_col_spacings(5);
-	$dialog->vbox->add($table);
-	
-	my $dir_label = Gtk2::Label->new('Directory:');
-	my $name_label = Gtk2::Label->new('Name:');
-	my $dir_entry = Gtk2::Entry->new();
-	$dir_entry->signal_connect(activate => sub { $dialog->response('ok') });
-	my $name_entry = Gtk2::Entry->new();
-	$name_entry->signal_connect(activate => sub { $dialog->response('ok') });
-	$table->attach_defaults($dir_label, 0,1, 0,1);
-	$table->attach_defaults($dir_entry, 1,2, 0,1);
-	$table->attach_defaults($name_label, 0,1, 1,2);
-	$table->attach_defaults($name_entry, 1,2, 1,2);
-
-	$dialog->show_all;
-	my ($name, $dir);
-	if ($dialog->run eq 'ok') {
-		$name = $name_entry->get_text;
-		$dir = $dir_entry->get_text;
-	}
-	$dialog->destroy;
-
-	if ($dir !~ /\S/) { $name = undef }
-	elsif ($name !~ /\S/) { $name = $dir }
+	my ($dir, $name) = @$val;
+	return undef unless $dir =~ /\S/;
+	$name = $dir unless $name =~ /\S/;
 	
 	return ($name, $dir);
 }
 
-=item C<prompt_browser_dialog>
-
-=cut
-
-sub prompt_browser_dialog {
-	my ($self, $type) = @_;
-	
-	my $title = ($type eq 'file_browser') ? "Choose file browser" : "Choose browser";
-	my $dialog = Gtk2::Dialog->new(
-		$title, $self->{window},
-	       	[qw/modal destroy-with-parent no-separator/],
-		'gtk-cancel' => 'cancel',
-		'gtk-ok'     => 'ok',
-	);
-	$dialog->set_resizable(0);
-	$dialog->set_border_width(5);
-	$dialog->set_icon($self->{window}->get_icon);
-	$dialog->set_default_response('ok');
-
-	my $label = Gtk2::Label->new(
-		($type eq 'file_browser')
-			? "Please enter a file browser"
-			: "Please enter a browser\nto open external links." );
-	$dialog->vbox->add($label);
-	
-	my $entry = Gtk2::Entry->new();
-	$entry->signal_connect(activate => sub { $dialog->response('ok') });
-	$entry->set_text(($type eq 'file_browser') ? "rox '\%s'" : "firefox '\%s'");
-	$dialog->vbox->add($entry);
-
-	$dialog->show_all;
-	my $browser = 
-		($dialog->run eq 'ok') ? $entry->get_text : undef ;
-	$dialog->destroy;
-	$browser = undef unless $browser =~ /\S/;
-
-	return $browser;
-}
-
 # "Search" dialog
 #
 # This dialog asks for a search query and then generates
