@@ -9,7 +9,7 @@ use Gtk2::Ex::HyperTextBuffer; #    "     "
 #use Gtk2::Ex::OpenWith;        # for file:// links
 use POSIX qw(strftime);
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 # TODO move more logic into Gtk2::Ex::HyperTextBuffer !
 
@@ -715,7 +715,7 @@ sub on_populate_popup {
 		$type = 'image';
 		$source = $pixbuf->{image_src};
 		$copy_string = 'Copy Image _Location';
-		$edit_string = '_Edit properties';
+		$edit_string = '_Edit source';
 		$code = sub {$self->edit_image_dialog($iter)};
 	}
 	else { return }
@@ -724,12 +724,10 @@ sub on_populate_popup {
 	$seperator->show;
 	$menu->prepend($seperator);
 	
-	if ($type eq 'link') { # TODO image support
-		my $edit_item = Gtk2::MenuItem->new($edit_string);
-		$edit_item->show;
-		$edit_item->signal_connect_swapped(activate => $code, $self);
-		$menu->prepend($edit_item);
-	}
+	my $edit_item = Gtk2::MenuItem->new($edit_string);
+	$edit_item->show;
+	$edit_item->signal_connect_swapped(activate => $code, $self);
+	$menu->prepend($edit_item);
 
 	my $copy_item = Gtk2::MenuItem->new($copy_string);
 	$copy_item->signal_connect(activate => sub {
@@ -1014,11 +1012,11 @@ sub parse_line {
 		$buffer->delete($lf, $iter);
 		$buffer->insert_with_tags_by_name($lf, $line, "head$h");
 	}
-	elsif ($line =~ /^\s*([\*\x{2022}]|(\d+|\w)\W*)\s+$/) {
-		# empty bullet or list item
+	elsif ($line =~ /^\s*(\W+|(\d+|\w)\W?)\s+$/) { # empty bullet or list item
+		# TODO check previous line for same pattern !
 		$buffer->delete($lf, $iter);
 	}
-	elsif ($line =~ /^(\s*(\d*|\w?)\W*\s+)/) { # auto indenting + lists
+	elsif ($line =~ /^(\s*(:?\W+|(\d*|\w?)\W?)\s+)/) { # auto indenting + lists
 		my ($indent, $number) = ($1, $2);
 		if (length $number) { # numbered list
 			$number = ($number =~ /\d/) ? $number+1 : chr(ord($number)+1);
@@ -1382,6 +1380,8 @@ If ITER is undefined the cursor is used.
 sub edit_link_dialog {
 	my ($self, $iter) = @_;
 	my $buffer = $self->{buffer};
+
+	# handle selections
 	my ($start, $end) = $buffer->auto_selection('link', $iter);
 
 	my ($text, $link) = ('', '');
@@ -1392,11 +1392,13 @@ sub edit_link_dialog {
 		$text = undef if $text =~ /\n/;
 	}
 
+	# Run the dialog
 	my $title = defined($start) ? 'Edit Link' : 'Insert Link';
-	
-	($text, $link) = $self->{app}->prompt_link_dialog($text, $link, $title);
-
-	return unless defined $text and ($text =~ /\S/ or $link =~ /\S/);
+	my $val = $self->{app}->run_prompt( $title,
+		['Text', 'Links to'], [$text, $link], $LINK_ICON, '_Link' )
+		or return;
+	($text, $link) = @$val;
+	return unless $text =~ /\S/ or $link =~ /\S/;
 	
 	# both entries default to the other
 	$link = $text unless $link =~ /\S/;
@@ -1424,8 +1426,25 @@ sub edit_link_dialog {
 
 =cut
 
-sub edit_image_dialog {
-	# TODO 
+sub edit_image_dialog { # TODO add 'browse' button
+	my ($self, $iter) = @_;
+	my $pixbuf = $iter->get_pixbuf;
+	return unless $pixbuf and defined $pixbuf->{image_src};
+	
+	my $src = $pixbuf->{image_src};
+	my $val = $self->{app}->run_prompt( 'Edit Image Source',
+		['Source'], [$src], $LINK_ICON, '_Link' )
+		or return;
+
+	my ($new_src) = @$val;
+	return unless $new_src =~ /\S/ and $new_src ne $src;
+
+	my $buffer = $self->{buffer};
+	my $end = $iter->copy;
+	$end->forward_char;
+	$buffer->delete($iter, $end);
+	$buffer->place_cursor($iter);
+	$self->insert_image($new_src);
 }
 
 1;
