@@ -4,6 +4,19 @@ use strict;
 
 our $VERSION = 0.01;
 
+our %TAG_ALIASES = ( # FIXME what to use for strike ??
+	B         => 'bold',
+	C         => 'verbatim',
+	I         => 'italic',
+	L         => 'link',
+	U         => 'underline',
+	bold      => 'B',
+	verbatim  => 'C',
+	italic    => 'I',
+	link      => 'L',
+	underline => 'U',
+);
+
 =head1 NAME
 
 ... - simple module
