@@ -3,7 +3,7 @@ package Zim::Formats::Wiki;
 use strict;
 no warnings;
 
-our $VERSION = '0.10';
+our $VERSION = '0.12';
 
 # TODO: some tags can be nested: email links for example
 # TODO: header in the middle of a para doesn't get recognized
@@ -131,11 +131,13 @@ sub parse_styles { # parse blocks of bold, italic and underline
 		elsif (/^\'\'(.+)\'\'$/) { ['verbatim',  {}, $1] }
 		elsif (/^\*\*(.+)\*\*$/) { ['bold',      {}, $1] }
 		elsif (/^\/\/(.+)\/\/$/) { ['italic',    {}, $1] }
+		elsif (/^\~\~(.+)\~\~$/) { ['strike',    {}, $1] }
 		elsif (/^\_\_(.+)\_\_$/) { ['underline', {}, $1] }
 	} split /(
 		\'\'.+?\'\' |
 		\*\*.+?\*\* |
 		(?<!\:)\/\/.+?\/\/ |
+		\~\~.+?\~\~ |
 		__.+?__
 	)/x, $text;
 }
@@ -171,41 +173,45 @@ sub _save_tree {
 			next;
 		}
 		
-		my ($tag, $meta, @node) = @$node;
+		my ($tag, $meta) = @$node[0, 1];
+		my $text = join '', @$node[2 .. $#$node]; # non-recursive tags
 		if ($tag eq 'Para') {
 			$class->_save_tree($node);
 		}
 		elsif ($tag eq 'Verbatim')  {
-			s/^/    /mg for @node;
-			print @node;
+			$text =~ s/^/\t/mg;
+			print $text;
 		}
 		elsif ($tag =~ /^head(\d)$/) {
 			my $n = 7 - $1;
-			print ( ('='x$n)." $node[0] ".('='x$n)."\n" );
+			print ( ('='x$n)." $text ".('='x$n)."\n" );
 		}
 		elsif ($tag eq 'image') {
 			print '{{'.$$meta{src}.'}}';
 		}
 		elsif ($tag eq 'link') {
 			my $to = $$meta{to};
-			$to =~ s/^mailto:// unless $node[0] =~ /^mailto:/;
+			$to =~ s/^mailto:// unless $text =~ /^mailto:/;
 			print (
-				($to ne $node[0]) ? "[[$to|$node[0]]]" :
+				($to ne $text) ? "[[$to|$text]]" :
 				($to =~ m#^\w+://|^(?:mailto:)?\S+\@\S+\.\w+#)
 					? $to : "[[$to]]" );
 		}
 		# per line markup for remaining tags ...
 		elsif ($tag eq 'bold') {
-			print map { /\S/ ? "**$_**" : $_} split /(\n)/, $node[0]
+			print map { /\S/ ? "**$_**" : $_} split /(\n)/, $text
 		}
 		elsif ($tag eq 'italic') {
-			print map { /\S/ ? "//$_//" : $_} split /(\n)/, $node[0]
+			print map { /\S/ ? "//$_//" : $_} split /(\n)/, $text
+		}
+		elsif ($tag eq 'strike') {
+			print map { /\S/ ? "~~$_~~" : $_} split /(\n)/, $text
 		}
 		elsif ($tag eq 'underline') {
-			print map { /\S/ ? "__$_\__" : $_} split /(\n)/, $node[0]
+			print map { /\S/ ? "__$_\__" : $_} split /(\n)/, $text
 		}
 		elsif ($tag eq 'verbatim') {
-			print map { /\S/ ? "''$_''" : $_} split /(\n)/, $node[0]
+			print map { /\S/ ? "''$_''" : $_} split /(\n)/, $text
 		}
 		else { die "Unkown tag in wiki parse tree: $tag\n" }
 		
