@@ -2,7 +2,11 @@ package Zim::Page;
 
 use strict;
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
+
+use overload
+	'""' => sub { $_[0]->{name} }, # \&name results in bugs ?
+	fallback => 'TRUE' ;
 
 =head1 NAME
 
@@ -21,6 +25,11 @@ interfaces supported by an individual object can change more dynamicly.
 This is a general base class that does not support any interfaces
 itself and cannot be used on it's own.
 
+=head1 OVERLOAD
+
+This class overloads the '""' operator, so the string version of an object
+is the page name.
+
 =head1 METHODS
 
 =over 4
