@@ -4,8 +4,9 @@ use strict;
 use Carp;
 use UNIVERSAL qw/isa/;
 use File::Spec;
+use Zim::File;
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 $Carp::CarpLevel = 1;
 
@@ -63,26 +64,21 @@ sub new {
 	return $self unless length $dir;
 	
 	# load config file
-	my $conf = File::Spec->catfile($dir, '.zim.config');
-	if (-f $conf and -r _) {
-		$self->{file} = $conf;
-		open CONFIG, "<$conf" or die "Could not open: $conf\n";
-		while (<CONFIG>) {
-			chomp;
-			next unless /\S/;
-			my ($opt, $val) = split '=', $_, 2;
-			if ($opt =~ /^:/) { # opt is namespace
-				my ($class, $arg) = split '=', $val, 2;
-				my @arg = split ',', $arg if defined $arg;
-				$self->add_child($opt, $class, @arg);
-			}
-			else { $self->{config}{$opt} = $val }
+	my $conf = Zim::File->new($dir, '.zim.config');
+	for ($conf->read) {
+		chomp;
+		next unless /\S/;
+		my ($opt, $val) = split '=', $_, 2;
+		if ($opt =~ /^:/) { # opt is namespace
+			my ($class, $arg) = split '=', $val, 2;
+			my @arg = split ',', $arg if defined $arg;
+			$self->add_child($opt, $class, @arg);
 		}
-		close CONFIG;
+		else { $self->{config}{$opt} = $val }
 	}
 	
 	# default config
-	$self->add_child(':', 'File', $dir, $format)
+	$self->add_child(':', 'Files', $dir, $format)
 		unless defined $self->{':'};
 
 	return $self;
@@ -239,7 +235,7 @@ sub export {
 	my $namespace = ($page =~ /:$/) ? $page : '';
 	#print STDERR "export root: $opts{root}\n";
 	my $exporter = ref($opts{root}) ? $opts{root} :
-		Zim::Repository::File->new(undef, '', $opts{root}, $opts{format});
+		Zim::Repository::Files->new(undef, '', $opts{root}, $opts{format});
 	#print STDERR "export template: $opts{template}\n";
 	$exporter->{_template} = $opts{template}; # FIXME better interface for this - do not confuse with the template for new pages
 	
@@ -471,6 +467,6 @@ modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<Zim>, L<Zim::Repository::File>, L<Zim::Repository::DBI>
+L<Zim>, L<Zim::Repository::Files>, L<Zim::Repository::DBI>
 
 =cut
